function [all_features,seg] = get_features(header,recordings,segment,HSMM,do_suppress) %get_ECGLeads_features

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: Extract features from PCG recordings
% Inputs:
% 1. Header data
% 2. The recordings
%
% Outputs:
% features
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    age_group=get_age(header);
    
    if strcmpi(age_group,'neonate')
        age=0.5;
    elseif strcmpi(age_group,'infant')
        age=6;
    elseif strcmpi(age_group,'child')
        age=6*12;
    elseif strcmpi(age_group,'adolescent')
        age=15*12;
    elseif strcmpi(age_group,'young adult')
        age=20*12;
    else
        age=0;
    end
    
    sex_group=get_sex(header);
    if strcmpi(sex_group,'male')
        sex=0;
    elseif strcmpi(sex_group,'female')
        sex=1;
    else
        sex=2;
    end
    
    height=get_height(header);
    weight=get_weight(header);
    
    recording_locations={'AV','MV','PV','TV'};
    num_recording_locations=length(recording_locations);
    
    recording_features=nan([num_recording_locations,214]);
    
    locations=get_locations(header);
    num_locations=length(locations);
    seg = {};
    
    for j=1:num_locations
        features = [];
        ind=strmatch(locations{j},recording_locations);

        if isempty(ind)
            continue
        end

        [segm_recording,downsample_recording] = preproc_signal(recordings{j},1000);
        
        if do_suppress
            supr_recording = suppress_breathing(downsample_recording,250,0.7,false);
        end

        if isempty(segment)
            if isempty(HSMM.B)
                [s_locs,r_ms] = localize_heartsounds(segm_recording,4,60,110,0.4,120);
                [s1_locs, s2_locs] = separate_heartsounds(s_locs);
                [s1_start, s1_stop] = get_sound_events(s1_locs,r_ms,80,0.6,0.9);
                [s2_start, s2_stop] = get_sound_events(s2_locs,r_ms,80,0.6,0.9);
                [sys_start,sys_stop] = get_inter_segments(s1_stop,s2_start,1.5,0.5);
                [dia_start,dia_stop] = get_inter_segments(s2_stop,s1_start,1.5,0.5);
            else
            [s1_start,s1_stop,s2_start,s2_stop,sys_start,sys_stop,dia_start,dia_stop]=...
                segment_HSMM(HSMM.B,HSMM.pi_v,HSMM.obs,segm_recording,1000);
            end
        else
            seg = segment{j};
            if any(cellfun('isempty',seg))
                continue
            end
            s1_start_p = seg{1}(:,1);
            s1_stop_p = seg{1}(:,2);
            s2_start_p = seg{2}(:,1);
            s2_stop_p = seg{2}(:,2);
            sys_start_p = seg{3}(:,1);
            sys_stop_p = seg{3}(:,2);
            dia_start_p = seg{4}(:,1);
            dia_stop_p = seg{4}(:,2);

            s1_start = s1_start_p(s1_start_p>1000 & s1_start_p<length(downsample_recording) & s1_stop_p<length(downsample_recording));
            s1_stop = s1_stop_p(s1_start_p>1000 & s1_start_p<length(downsample_recording) & s1_stop_p<length(downsample_recording));
            s2_start = s2_start_p(s2_start_p>1000 & s2_start_p<length(downsample_recording) & s2_stop_p<length(downsample_recording));
            s2_stop = s2_stop_p(s2_start_p>1000 & s2_start_p<length(downsample_recording) & s2_stop_p<length(downsample_recording));
            sys_start = sys_start_p(sys_start_p>1000 & sys_start_p<length(downsample_recording) & sys_stop_p<length(downsample_recording));
            sys_stop = sys_stop_p(sys_start_p>1000 & sys_start_p<length(downsample_recording) & sys_stop_p<length(downsample_recording));
            dia_start = dia_start_p(dia_start_p>1000 & dia_start_p<length(downsample_recording) & dia_stop_p<length(downsample_recording));
            dia_stop = dia_stop_p(dia_start_p>1000 & dia_start_p<length(downsample_recording) & dia_stop_p<length(downsample_recording));
        end

        seg{j} = {[s1_start;s1_stop]',[s2_start;s2_stop]',[sys_start;sys_stop]',[dia_start;dia_stop]'};

        [c,l] = wavedec(supr_recording,2,'db6');
        det_1 = detcoef(c,l,1);
        det_1 = resample(det_1,length(supr_recording),length(det_1));
        det_1 = det_1/max(det_1);

        [s1_len,K_s1,rms_s1,Fl_s1,Fh_s1,lyap_s1,lyap_wt_s1,zcr_s1,wt_s1,hi_lo_s1,hi_all_s1,freq_kurt_s1] = get_segment_feature(supr_recording,det_1,s1_start,s1_stop);
        [s2_len,K_s2,rms_s2,Fl_s2,Fh_s2,lyap_s2,lyap_wt_s2,zcr_s2,wt_s2,hi_lo_s2,hi_all_s2,freq_kurt_s2] = get_segment_feature(supr_recording,det_1,s2_start,s2_stop);
        [sys_len,K_sys,rms_sys,Fl_sys,Fh_sys,lyap_sys,lyap_wt_sys,zcr_sys,wt_sys,hi_lo_sys,hi_all_sys,freq_kurt_sys] = get_segment_feature(supr_recording,det_1,sys_start,sys_stop);
        [dia_len,K_dia,rms_dia,Fl_dia,Fh_dia,lyap_dia,lyap_wt_dia,zcr_dia,wt_dia,hi_lo_dia,hi_all_dia,freq_kurt_dia] = get_segment_feature(supr_recording,det_1,dia_start,dia_stop);

        b2b_times = diff(s1_start);
        features = [features,rms(b2b_times)];
        
        [len_mean,len_std] = feature_mean_std(s1_len,s2_len,sys_len,dia_len);
        features = [features,len_mean,len_std];

        % Low frequency
        % mean, std at max, max freq
        [freq_mean,freq_std,freq_max] = freq_features(Fl_s1,Fl_s2,Fl_sys,Fl_dia);
        features = [features,freq_mean,freq_std,freq_max];
        
        % High frequency
        % mean, std at max, max freq
        [freq_mean,freq_std,freq_max] = freq_features(Fh_s1,Fh_s2,Fh_sys,Fh_dia);
        features = [features,freq_mean,freq_std,freq_max];
        
        % kurtosis
        [kurt_mean,kurt_std] = feature_mean_std(K_s1,K_s2,K_sys,K_dia);
        features = [features,kurt_mean,kurt_std];

        % spectral kurtosis
        [freq_kurt_mean,freq_kurt_std] = feature_mean_std(freq_kurt_s1,freq_kurt_s2,freq_kurt_sys,freq_kurt_dia);
        features = [features,freq_kurt_mean,freq_kurt_std];

        % rms
        [rms_mean,rms_std] = feature_mean_std(rms_s1,rms_s2,rms_sys,rms_dia);
        features = [features,rms_mean,rms_std];

        % zerocrossrate
        [zcr_mean,zcr_std] = feature_mean_std(zcr_s1,zcr_s2,zcr_sys,zcr_dia);
        features = [features,zcr_mean,zcr_std];

        % wavelet detail 1 energy
        [wt_mean,wt_std] = feature_mean_std(wt_s1,wt_s2,wt_sys,wt_dia);
        features = [features,wt_mean,wt_std];

        % hi lo freq ratio
        [hi_lo_mean,hi_lo_std] = feature_mean_std(hi_lo_s1,hi_lo_s2,hi_lo_sys,hi_lo_dia);
        features = [features,hi_lo_mean,hi_lo_std];

        % hi all freq ratio mean std
        [hi_all_mean,hi_all_std] = feature_mean_std(hi_all_s1,hi_all_s2,hi_all_sys,hi_all_dia);
        features = [features,hi_all_mean,hi_all_std];

        % lyapunov exponent
        [lyap_mean,lyap_std] = feature_mean_std(lyap_s1,lyap_s2,lyap_sys,lyap_dia);
        features = [features,lyap_mean,lyap_std];

        % lyapunov exponent wavelet detail 1
        [lyap_wt_mean,lyap_wt_std] = feature_mean_std(lyap_wt_s1,lyap_wt_s2,lyap_wt_sys,lyap_wt_dia);
        features = [features,lyap_wt_mean,lyap_wt_std];
        
        % total signal features
        sm_env = mean(abs(stft(supr_recording,1000,"Window",hann(30),"OverlapLength",29)));
        [aut,lags]=xcorr(sm_env,3000,'normalized');
        peak = max(aut(lags>300 & lags<2500));
        total_kurt = kurtosis(supr_recording);
        [psd,f] = pwelch(supr_recording,[],[],[],1000);
        lo_freq_ratio = sum(psd(f>24 & f<144))/sum(psd);
        mi_freq_ratio = sum(psd(f>144 & f<200))/sum(psd);
        hi_freq_ratio = sum(psd(f>200))/sum(psd);
        total_features = [peak,total_kurt,lo_freq_ratio,mi_freq_ratio,hi_freq_ratio];
        features = [features,total_features];

        recording_features(ind,:) = features;
    end
    
%     recording_features=recording_features(:);
    
    all_features=[mean(recording_features,'omitnan'),...
                  median(recording_features,'omitnan'),...
                  std(recording_features,'omitnan'),...
                  var(recording_features,'omitnan'),...
                  max(recording_features),...
                  min(recording_features),...
                  age sex height weight];
end

function age_group=get_age(header)
    age_group=header(startsWith(header,'#Age'));
    age_group=strsplit(age_group{1},':');
    age_group=strtrim(age_group{2});
end

function sex=get_sex(header)
    sex=header(startsWith(header,'#Sex'));
    sex=strsplit(sex{1},':');
    sex=strtrim(sex{2});
end

function height=get_height(header)
    height=header(startsWith(header,'#Height'));
    height=strsplit(height{1},':');
    height=strtrim(height{2});
    height=str2double(height);
    
    if isnan(height)
        height=0;
    end
end

function weight=get_weight(header)
    weight=header(startsWith(header,'#Weight'));
    weight=strsplit(weight{1},':');
    weight=strtrim(weight{2});
    weight=str2double(weight);
    
    if isnan(weight)
        weight=0;
    end
end

function locations=get_locations(header)
    num_locations=strsplit(header{1},' ');
    num_locations=str2double(num_locations{2});
    
    locations={};
    
    for j=2:num_locations+1
        current_line=strsplit(header{j},' ');
        locations{j-1}=current_line{1};
    end
end
