function [starts,stops] = get_inter_segments(event1_loc,event2_loc,up_th,low_th)
%GET_INTER_SEGMENTS Gets the start and stop events between two event lists
%   Returned values:
%       - start events of inter-segments
%       - stop events of inter-segments
%   Input parameters:
%       - event 1 locations
%       - event 2 locations
%       - outlier upper threshold
%       - outlier lower threshold

    mins = [];
    locs = [];
    % calculate smallest difference between events
    if ~isempty(event2_loc)
        for k=1:length(event1_loc)
            [mins(k),locs(k)]=min(abs(event2_loc-event1_loc(k)));
        end
    end
    
    % remove "outliers" based on mean and thresholds
    m_mins = mean(mins);
    rem_outlier = (mins<up_th*m_mins & mins>low_th*m_mins);
    stoploc = locs(rem_outlier);
    p_starts = event1_loc(rem_outlier);
    p_stops = event2_loc(stoploc);
    test = p_starts<p_stops;
    starts = p_starts(test);
    stops = p_stops(test);
end