function [B_matrix, pi_vector, total_obs_distribution] = train_segmentation(input_dir)

    Fs = 1000;

    disp("["+datestr(now,'HH:MM:SS')+"]: Preprocessing recordings...")
    sound_files = dir(input_dir+"/*.wav");
    tr_audio = {};
    for k=1:length(sound_files)
%         disp(string(files(k).name))
        [audio,~] = audioread(string(sound_files(k).folder)+"/"+string(sound_files(k).name));
        sup_audio = preproc_signal(audio,Fs);
        tr_audio{k} = sup_audio;
    end
    
    disp("["+datestr(now,'HH:MM:SS')+"]: Preprocessing segment labels...")
    annot_files = dir(input_dir+"/*.tsv");
    tr_annots = {};
    invalid = [];
    for k=1:length(annot_files)
%         disp(string(annot_files(k).name))
        annot = readtable(string(annot_files(k).folder)+"/"+string(annot_files(k).name),...
            'ReadVariableNames', false, 'FileType', 'text');
        annot = table2array(annot);
        starts = annot(:,1);
        if length(annot)>1
            has_s1 = sum(annot(:,3)==1) > 0;
            has_s2 = sum(annot(:,3)==3) > 0;
            if has_s1 && has_s2
                s1_locs = round(starts(annot(:,3)==1)*Fs);
                s2_locs = round(starts(annot(:,3)==3)*Fs);
                max_loc = length(tr_audio{k})+Fs;
                tr_annots{k,1} = s1_locs(s1_locs>Fs & s1_locs<max_loc);
                tr_annots{k,2} = s2_locs(s2_locs>Fs & s2_locs<max_loc);
            else
                warning("Skipping invalid annotation file: "+string(annot_files(k).name))
                invalid = [invalid, k];
            end
        else
            warning("Skipping invalid annotation file: "+string(annot_files(k).name))
            invalid = [invalid, k];
%             tr_annots{k,1} = 0;
%             tr_annots{k,2} = 0;
        end
    end
    tr_audio(invalid) = [];

    disp("["+datestr(now,'HH:MM:SS')+"]: Training...")
    [B_matrix, pi_vector, total_obs_distribution] =...
        trainSpringerSegmentationAlgorithm(tr_audio, tr_annots, Fs, false);

end

